// Georgui Rekouts, Nov, 1994
// treelist.cpp - Implementation of TTSCTreeList

#include "treelist.h"

// ----------------------------------- constructors and destructor
TTSCTreeList::TTSCTreeList(TWindow *parent, int id, int x, int y,
  int w, int h, TModule *module) :
  TListBox(parent, id, x, y, w, h, module)
{
  Attr.Style |= LBS_NOTIFY | LBS_OWNERDRAWVARIABLE | LBS_NOINTEGRALHEIGHT;
  Attr.Style &= ~LBS_HASSTRINGS;
  CommonInit();
}

TTSCTreeList::TTSCTreeList(TWindow *parent, int resId, TModule *module) :
  TListBox(parent, resId, module)
{
  CommonInit();
}

void TTSCTreeList::CommonInit()
{
  Font     = NULL;
  ViewList = new TViewList;
  NodeRoot = new TNode;
  SetIndentUnits(30);
}

TTSCTreeList::~TTSCTreeList()
{
  delete NodeRoot;
  FreeViewList(ViewList);
  if (Font)
	 delete Font;
}

//----------------------------------------------- ownerdraw methods
void TTSCTreeList::MeasureItem(MEASUREITEMSTRUCT &measureInfo)
{
  TNode *node = GetNodeAtIndex(measureInfo.itemID);
  if (node == NULL)
	 return;
  TNodeView *view = node->GetNodeView();
  if (view == NULL)
	 return;

  TSize size(-1, -1);
  view->GetSize(node, Font, size);

  if (size.cx != -1)
	 measureInfo.itemWidth = size.cx;
  if (size.cy != -1)
	 measureInfo.itemHeight = size.cy;
}

void TTSCTreeList::ODADrawEntire(DRAWITEMSTRUCT &drawInfo)
{
  //............ draw the item
  TNode *node = GetNodeAtIndex(drawInfo.itemID);
  if (node == NULL)
	 return;
  TNodeView *view = node->GetNodeView();
  if (view == NULL)
	 return;
  view->Paint( drawInfo, node, Font, GetIndentUnits());
}

void TTSCTreeList::ODAFocus(DRAWITEMSTRUCT &drawInfo)
{
	ODADrawEntire( drawInfo );
}

void TTSCTreeList::ODASelect(DRAWITEMSTRUCT &drawInfo)
{
	ODADrawEntire( drawInfo );
}

//--------------------------------------------- Response table
DEFINE_RESPONSE_TABLE1(TTSCTreeList, TListBox )
  EV_WM_SETFONT,
  EV_WM_MEASUREITEM,
  EV_NOTIFY_AT_CHILD(LBN_DBLCLK, EvDblClk),
END_RESPONSE_TABLE;

//--------------------------------------------------------------- EvSetFont
void TTSCTreeList::EvSetFont( HFONT hFont, BOOL bRedraw )
{
  TListBox::EvSetFont( (HFONT)hFont, bRedraw );
  if( Font )
  {
	 delete Font;
  }
  Font = new TFont((HFONT)hFont);
}

//--- EvDblClk-Toggles the parent's state, Notify the parent of a double-click
void TTSCTreeList::EvDblClk()
{
  TNode *node = GetSelectedNode();
  if (node && node->IsParent())
  {
	 int index = GetSelIndex();
	 if( node->IsOpen() )
	 {
		CloseChildList( index );
	 }
	 else
	 {
		OpenChildList( index );
	 }
  }
  //........... allow parent to handle dblclk
  DefaultProcessing();
}

// ----------------------------------- node and nodeList management
int TTSCTreeList::AddNodeList(TNodeList *nodeList, int indentLevel,
  int insertAfterIndex)
{
  if (nodeList == NULL)
	 return insertAfterIndex;
  int count = 0;
  TNodeListIterator iter(*nodeList);

  //.......... for each item in the list
  while (iter)
  {
	 TNode *node = iter.Current();
	 //............................ add item into the listbox
	 int index = InsertString((char far *)node, insertAfterIndex);
	 node->SetIndentLevel(indentLevel);
	 //............................ add children if necessary
	 if (node->IsParent() && node->IsOpen())
		AddNodeList(node->GetChildList(), indentLevel+1, index+1);
	 count++;
	 iter++;
  }
  return insertAfterIndex+count;
}

//......................................... Open Child List
void TTSCTreeList::OpenChildList( int index )
{
  TNode *node = GetNodeAtIndex( index );
  node->SetOpen( TRUE );

  TNodeList *childList = node->GetChildList();
  AddNodeList(childList, node->GetIndentLevel()+1, index+1);
}

//......................................... Close Child List
void TTSCTreeList::CloseChildList( int index )
{
  TNode *node = GetNodeAtIndex( index );
  node->SetOpen(FALSE);

  index++;
  if (index >= GetCount())    // at end of listbox
	 return;

  int targetDeleteIndent = node->GetIndentLevel()+1;

  node = GetNodeAtIndex(index);
  int currentIndentLevel = node->GetIndentLevel();

  while (targetDeleteIndent <= currentIndentLevel)
  {
	 if (node->IsOpen())
		CloseChildList( index );

	 int count = DeleteString(index);
	 if (index >= count)
		break;              // no more items

	 node = GetNodeAtIndex(index);
	 currentIndentLevel = node->GetIndentLevel();
  }
}

//......................................... Get Selected Node
TNode *TTSCTreeList::GetSelectedNode()
{
  int index = GetSelIndex();
  return GetNodeAtIndex(index);
}

//......................................... Get Node At Index
TNode *TTSCTreeList::GetNodeAtIndex(int index)
{
  if (index < 0 || index >= GetCount())
	 return NULL;
  return (TNode *)GetItemData(index);
}

//......................................... Add Node
TNode *TTSCTreeList::AddNode(TNode *parent, TNode *node, TNodeView *view)
{
	if( parent && node )
	{
		node->SetParent(parent);
		node->SetNodeView(view);
		if (parent->IsParent())
		{
			parent->GetChildList()->Add(node);
		}
		else
		{
			TNodeList *nodeList = new TNodeList;
			nodeList->Add(node);
			parent->SetChildList(nodeList);
		}
	}
	return node;
}

//......................................... Remove Node
TNode *TTSCTreeList::RemoveNode( int index )
{
  TNode *node = GetNodeAtIndex( index );
  TNode *parent = node->GetParent();

  parent->GetChildList()->Detach( node );
  if( parent->GetChildList()->GetItemsInContainer() == 0 )
  {
	  parent->SetOpen(FALSE);
  }
  return node;
}

//......................................... Update
void TTSCTreeList::Update()
{
  TNode *root = GetNodeRoot();
  SaveVisualState();
  ClearList();
  AddNodeList(root->GetChildList(), 0, 0);
  RestoreVisualState();
}

void TTSCTreeList::SaveVisualState()
{
  SendMessage(WM_SETREDRAW, FALSE);
  TopIndex = GetTopIndex();
}

void TTSCTreeList::RestoreVisualState()
{
  SetTopIndex(TopIndex);
  SendMessage(WM_SETREDRAW, TRUE);
}

//...................................... view management
TNodeView *TTSCTreeList::AddView(TNodeView *view)
{
  GetViewList()->Add(view);
  return view;
}

//-------------------------------------- Free View List
void FreeViewList(TViewList *viewList)
{
  if( viewList )
  {
	  TViewListIterator iter(*viewList);
	  while (iter)
	  {
			delete iter.Current();
			iter++;
	  }
	  delete viewList;
  }
}

