// Georgui Rekouts, Nov 1994
// treelist.h - TTSCTreeList, TNode, TNodeView, TStringNode, TStringView                                       //
            
#ifndef __TREELIST_H
#define __TREELIST_H

#include <owl\gdiobjec.h>
#include <owl\listbox.h>
#include <classlib\listimp.h>
#include <windowsx.h>
#include <string.h>

class TNode;
typedef TListImp<TNode *>         TNodeList;
typedef TListIteratorImp<TNode *> TNodeListIterator;

// =========================================================== TNodeView
// Provides the user-interface component of each element in the outliner.                                                      //
// Given a node, the view will know how to paint the node.

class TNodeView
{
  public:
				 TNodeView();
	 virtual ~TNodeView();

	 //........ needed operators for TListImp
	 TNodeView(const TNodeView &);
	 TNodeView& operator = (const TNodeView &);
	 int operator == (const TNodeView &) const;

	 //........ derived classes should override these
	 virtual void Paint(DRAWITEMSTRUCT &drawInfo, TNode *node, TFont *, int indentUnits);
	 virtual void GetSize(TNode *node, TFont *, TSize &);

	 //........ bitmaps
	 void     SetEmptyBitmap(TBitmap *);
	 TBitmap *GetEmptyBitmap() const       { return BmEmpty; }
	 void     SetOpenBitmap(TBitmap *);
	 TBitmap *GetOpenBitmap() const        { return BmOpen; }
	 void     SetCloseBitmap(TBitmap *);
	 TBitmap *GetCloseBitmap() const       { return BmClose; }
	 void     SetNodeBitmap(TBitmap *);
	 TBitmap *GetNodeBitmap() const        { return BmNode; }

  protected:
	 void DrawBitmap(TDC &dc, TBitmap *bitmap, TPoint point);
	 int GetFontHeight(HFONT hFont);
	 TBitmap *GetBitmap(TNode *);

	 TBitmap *BmEmpty;
	 TBitmap *BmOpen;
	 TBitmap *BmClose;
	 TBitmap *BmNode;
};

typedef TIListImp<TNodeView>         TViewList;
typedef TIListIteratorImp<TNodeView> TViewListIterator;


// ============================================================== TNode
// Base class for all items to be inserted into the outliner.

class TNode
{
  public:
					TNode( void* NodeData = 0 );
	 virtual   ~TNode();

	 //............. needed operators for TListImp
	 TNode(const TNode &);
	 TNode& operator = (const TNode &);
	 int operator == (const TNode &) const;

	 BOOL       IsParent() const           { return GetChildList() != NULL; }
	 BOOL       IsOpen() const             { return BIsOpen; }
	 void       SetOpen(BOOL isOpen)       { BIsOpen = isOpen; }

	 TNode     *GetParent() const          { return Parent; }
	 void       SetParent(TNode *parent);

	 void      *GetNodeData() const        { return NodeData; }
	 void       SetNodeData(void *nodeData)
														{ NodeData = nodeData; }

	 TNodeList *GetChildList() const       { return ChildList; }
	 void       SetChildList(TNodeList *childList);

	 int        GetIndentLevel() const
														{ return IndentLevel; }
	 void       SetIndentLevel(int indentLevel)
														{ IndentLevel = indentLevel; }

	 TNodeView *GetNodeView() const        { return View; }
	 void       SetNodeView(TNodeView *view)
														{ View = view; }

  protected:
	 TNode     *Parent;
	 TNodeList *ChildList;
	 TNodeView *View;

	 int        IndentLevel;
	 BOOL       BIsOpen;
	 void      *NodeData;
};

// ========================================================= TStringView
// Displays the string, calls base class to display bitmap and connection lines

class TStringView : public TNodeView
{
  public:
	 //........ derived classes should override these
	 virtual void Paint(DRAWITEMSTRUCT &drawInfo, TNode *node, TFont *,
		int indentUnits);
	 virtual void GetSize(TNode *node, TFont *, TSize &);
};

// ========================================================= TStringNode
// Use it as a base class if you need to store more than a string or
// use void NodeData pointer as a alternative ( not type save ! )

class TStringNode : public TNode
{
  public:
	 TStringNode(char *str);
	 TStringNode(string str);
	~TStringNode();

	 TStringNode(const TStringNode &);
	 TStringNode& operator = (const TStringNode &);
	 int operator == (const TStringNode &) const;

	 string  GetString() const       { return ItemString; }
	 void    SetString(string &str)  { ItemString = str; }
  protected:
	 string ItemString;
};

// ========================================================= TTSCTreeList
// An owner-drawn listbox that displays tree structures.

class TTSCTreeList : public TListBox
{
  public:
	 //................constructors and destructor
	 TTSCTreeList(TWindow *parent, int id, int x, int y, int w, int h,
		TModule *module = 0);
	 TTSCTreeList(TWindow *parent, int resId, TModule *module = 0);
	~TTSCTreeList();

	 //................owner drawn methods
	 void MeasureItem(MEASUREITEMSTRUCT &measureInfo);
	 void ODADrawEntire(DRAWITEMSTRUCT &drawInfo);
	 void ODAFocus     (DRAWITEMSTRUCT &drawInfo);
	 void ODASelect    (DRAWITEMSTRUCT &drawInfo);

	 //................ treelist manupulation functions
	 TNode     *GetSelectedNode();
	 TNode     *GetNodeAtIndex(int);

	 TNode     *AddNode(TNode *parent, TNode *node, TNodeView *);
	 TNode     *RemoveNode( int index );

	 void       Update();

	 TNode     *GetNodeRoot() const        { return NodeRoot; }
	 void       SetNodeRoot( TNode* nodeRoot )
														{ NodeRoot = nodeRoot; }

	 void       OpenChildList( int index);
	 void       CloseChildList( int index);

	 int        GetIndentUnits() const     { return IndentUnits; }
	 void       SetIndentUnits(int units)  { IndentUnits = units; }

	 TNodeView *AddView(TNodeView *view);
	 TViewList *GetViewList()              { return ViewList; }

	 DECLARE_RESPONSE_TABLE(TTSCTreeList);
  protected:
	 // .............messages handled by treelist
	 void EvDblClk();
	 void EvSetFont( HFONT hFont, BOOL bRedraw);

	 int  AddNodeList(TNodeList *, int indentLevel, int insertAfterIndex);

	 void SaveVisualState();
	 void RestoreVisualState();

	 int IndentUnits;
	 int TopIndex;

	 TViewList *ViewList;
	 TNode     *NodeRoot;
	 TFont     *Font;

  private:
	 void CommonInit();
};

//............. utility functions to help handle nodes & views lists
void FreeViewList(TViewList *);
void FreeNodeList(TNodeList *);

#endif
